@@ -15,7 +15,7 @@ static void prefetch_lookup(struct auth_request *auth_request,
 {
 	/* auth_request_set_field() should have already placed the userdb_*
 	   values to userdb_reply. */
-	if (auth_request->userdb_reply == NULL) {
+	if (!auth_request->userdb_prefetch_set) {
 		if (auth_request_get_auth(auth_request)->userdbs->next == NULL) {
 			/* no other userdbs */
 			if (auth_request->userdb_lookup) {
