@@ -374,7 +374,8 @@ int imap_sync_deinit(struct imap_sync_context *ctx,
 	ret = imap_sync_finish(ctx, TRUE);
 	imap_client_notify_finished(ctx->client);
 
-	if ((ctx->client->enabled_features & MAILBOX_FEATURE_QRESYNC) != 0)
+	if ((ctx->client->enabled_features & MAILBOX_FEATURE_QRESYNC) != 0 &&
+	    !ctx->client->nonpermanent_modseqs)
 		imap_sync_send_highestmodseq(ctx, sync_cmd);
 
 	if (array_is_created(&ctx->search_removes)) {
@@ -415,7 +416,8 @@ static int imap_sync_send_flags(struct imap_sync_context *ctx, string_t *str)
 	if (ctx->imap_flags & IMAP_SYNC_FLAG_SEND_UID)
 		str_printfa(str, "UID %u ", ctx->mail->uid);
 	if ((mailbox_get_enabled_features(ctx->box) &
-	     MAILBOX_FEATURE_CONDSTORE) != 0) {
+	     MAILBOX_FEATURE_CONDSTORE) != 0 &&
+	    !ctx->client->nonpermanent_modseqs) {
 		imap_sync_add_modseq(ctx, str);
 		str_append_c(str, ' ');
 	}
