@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2012 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2013 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -97,7 +97,8 @@ mail_index_alloc_cache_find(const char *mailbox_path, const char *index_dir,
 				else
 					match = rec;
 			}
-		} else if (mailbox_path != NULL && rec->mailbox_path != NULL) {
+		} else if (mailbox_path != NULL && rec->mailbox_path != NULL &&
+			   index_dir == NULL && rec->index_dir_ino == 0) {
 			if (strcmp(mailbox_path, rec->mailbox_path) == 0)
 				match = rec;
 		}
@@ -184,7 +185,8 @@ static void destroy_unrefed(bool all)
 		timeout_remove(&to_index);
 }
 
-static void index_removal_timeout(void *context ATTR_UNUSED)
+static void ATTR_NULL(1)
+index_removal_timeout(void *context ATTR_UNUSED)
 {
 	destroy_unrefed(FALSE);
 }
@@ -215,7 +217,7 @@ void mail_index_alloc_cache_unref(struct mail_index **_index)
 		mail_index_alloc_cache_list_free(list);
 	} else if (to_index == NULL) {
 		to_index = timeout_add(INDEX_CACHE_TIMEOUT*1000/2,
-				       index_removal_timeout, NULL);
+				       index_removal_timeout, (void *)NULL);
 	}
 }
 
